/*
 * Copyright (c) 2012 - 2020, Tim Düsterhus
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
define(["require", "exports"], function (require, exports) {
    "use strict";
    class NodePush {
        constructor() {
            this.initialized = false;
            this.connected = false;
            this.waitForInit = new Promise((resolve, reject) => {
                this.initResolve = resolve;
                this.initReject = reject;
            });
        }
        /**
         * Connect to the given host and provide the given signed authentication string.
         */
        init(host, connectData) {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.initialized) {
                    return;
                }
                this.initialized = true;
                try {
                    const socket = (yield new Promise((resolve_1, reject_1) => { require(["socket.io-client"], resolve_1, reject_1); }).then(__importStar)).default(host);
                    let token = undefined;
                    socket.on("connect", () => {
                        if (token === undefined) {
                            socket.emit("connectData", connectData);
                        }
                        else {
                            socket.emit("token", token);
                        }
                    });
                    socket.on("rekey", (newToken) => {
                        token = newToken;
                    });
                    socket.on("authenticated", () => {
                        this.connected = true;
                    });
                    socket.on("disconnect", () => {
                        this.connected = false;
                    });
                    this.initResolve(socket);
                }
                catch (err) {
                    console.log("Initializing nodePush failed:", err);
                    this.initReject(err);
                }
            });
        }
        getFeatureFlags() {
            return [
                "authentication",
                "target:channels",
                "target:groups",
                "target:users",
                "target:registered",
                "target:guest",
            ];
        }
        /**
         * Execute the given callback after connecting to the nodePush service.
         */
        onConnect(callback) {
            return __awaiter(this, void 0, void 0, function* () {
                const socket = yield this.waitForInit;
                socket.on("authenticated", () => {
                    callback();
                });
                if (this.connected) {
                    setTimeout(() => {
                        callback();
                    }, 0);
                }
            });
        }
        /**
         * Execute the given callback after disconnecting from the nodePush service.
         */
        onDisconnect(callback) {
            return __awaiter(this, void 0, void 0, function* () {
                const socket = yield this.waitForInit;
                socket.on("disconnect", () => {
                    callback();
                });
            });
        }
        /**
         * Execute the given callback after receiving the given message from the nodePush service.
         */
        onMessage(message, callback) {
            return __awaiter(this, void 0, void 0, function* () {
                if (!/^[a-zA-Z0-9-_]+\.[a-zA-Z0-9-_]+(\.[a-zA-Z0-9-_]+)+$/.test(message)) {
                    throw new Error("Invalid message identifier");
                }
                const socket = yield this.waitForInit;
                socket.on(message, (payload) => {
                    callback(payload);
                });
            });
        }
    }
    return new NodePush();
});
//# sourceMappingURL=NodePush.js.map